import os
import json


class FileGear(object):

    @staticmethod
    def fileContent(path, returnList=False):
        with open(path, 'r') as read_file:
            if returnList:
                return [line for line in read_file.read().split('\n')]
            return read_file.read()

    @staticmethod
    def get_files(path, extension=None, strip_extension=False):
        if not extension:
            return os.listdir(path)
        files = [file for file in os.listdir(path) if file.endswith(extension)]

        if strip_extension:
            splitext = os.path.splitext
            files = [splitext(filename)[0] for filename in files]

        return files

    @staticmethod
    def save_output(path, output, mode='w'):
        """
        Save content to a file

        :param output: Content to be written in a file
        :type output: str or list

        :param mode: 'w': write, 'a':append
        :type mode: str

        """
        os.makedirs(os.path.split(path)[0], exist_ok=True)
        try:
            with open(path, mode=mode) as out_file:
                if isinstance(output, list) or isinstance(output, tuple):
                    for index, line in enumerate(output):
                        if mode == 'a' and not index and os.stat(path).st_size:
                            """
                            not index: to match first list item
                            os.stat(path).st_size: to check if it is a new file
                            therefore it matches when:
                                mode is 'a'
                                index is 0
                                os.stat(path).st_size is not 0
                            """
                            out_file.write('\n')
                        out_file.write(line)
                        if index != (len(output) - 1):
                            out_file.write('\n')
                elif isinstance(output, dict):
                    out_file.write(json.dumps(output, indent=2))
                else:
                    if mode == 'a' and os.stat(path).st_size:
                        out_file.write('\n')
                    out_file.write(output.strip('\n'))
            return True
        except:
            """ Need to check open() exceptions """
            raise Exception
