from collections.abc import MutableSequence
from dracodes import FileGear as fg
from dracodes.ParserAspect.ParserLine import ParserLine
import re


class IndentParserGear(MutableSequence):

    def __init__(self, content):
        if content.endswith('txt'):
            self.content = fg.fileContent(content)
        else:
            self.content = content.split('\n')
        self._modules = list()
        self.parser = None
        self._defineBlocks()

    def _defineBlocks(self):
        """ function that iter through the file's content and divide the blocks"""
        indentations = [0]  # init the indentations control list
        depth = 0  # to control context
        parents = [None]  # to keep track of the parents
        # iter through the show run file
        for index, line in enumerate(self.content):
            indent = len(line) - len(line.lstrip())  # gets the actual indent
            # checks if this line's indentation is greater than the last one
            if indent > indentations[-1]:
                # keeps track of the parents
                parents.append(self._modules[index-1])
                depth += 1  # increments depth
                indentations.append(indent)
            # if the actual indent is lower than the last one
            elif indent < indentations[-1]:
                while indent < indentations[-1]:
                    depth -= 1
                    indentations.pop()  # pops the reference indentation
                    parents.pop()  # pops the actual parent
            module = ParserLine(line, depth, index)
            self._modules.append(module)
            if parents[-1] != None:  # if the actual parent is not None
                # then associate that parent to the child
                self._modules[-1].parent = parents[-1]
                # and add this child to parents
                parents[-1].childs.append(self._modules[-1])

    """ Auxiliary Functions """

    def surface(self, only_parents=True):
        """ returns every line in depth 0, can or not have childrens """
        return [c for c in self._modules if not c.depth and bool(c.childs) == only_parents]

    def delve(self, line, only_surface=False):
        """ function to filter based on regex or 'contains'

            if exact match is needed, just use regex => r'^whatever$'
        """
        input_pattern = re.compile(line)
        if only_surface:
            return [c for c in self._modules if input_pattern.match(str(c)) and c.depth == 0]
        return [c for c in self._modules if input_pattern.match(str(c))]
    """"""""""""""""""""""""""""""

    """ STANDARD FUNCTIONS """

    def __iter__(self):
        for module in self._modules:
            yield module

    def __setitem__(self, index, item):
        self._modules[index] = item

    def insert(self, index, item):
        self._modules[index] = item

    def __len__(self):
        return len(self._modules)

    def __getitem__(self, index):
        return self._modules[index]

    def __delitem__(self, index):
        del self._modules[index]
    """"""""""""""""""""""""""
